@@ -660,7 +660,7 @@ class TestSystemRaw(ExitCodeChecks):
     @mock.patch('os.system', side_effect=KeyboardInterrupt)
     def test_control_c(self, *mocks):
         try:
-            self.system("sleep 1 # wont happen")
+            self.system("sleep 1 # won't happen")
         except KeyboardInterrupt:  # pragma: no cove
             self.fail(
                 "system call should intercept "
@@ -721,7 +721,7 @@ class Negator(ast.NodeTransformer):
     """Negates all number literals in an AST."""
 
     def visit_Num(self, node):
-        node.n = -node.n
+        node.value = -node.value
         return node
 
     def visit_Constant(self, node):
